package edu.berkeley.qfat;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import java.util.*;
import edu.berkeley.qfat.geom.*;
import edu.berkeley.qfat.geom.Point;

public class MeshViewer implements GLEventListener, MouseListener, MouseMotionListener, KeyListener, MouseWheelListener  {

    public Mesh tile = new Mesh();
    public Mesh goal = new Mesh();
    public Matrix[] translations;
    public Mesh.Vert[] points;


    boolean alt = false;
    boolean shift = false;
    boolean control = false;

    public void mouseWheelMoved(MouseWheelEvent e) {
        tz -= e.getWheelRotation();
    }

    public void keyTyped(KeyEvent e)  { }
    public void keyPressed(KeyEvent e)  {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_CONTROL: control = true; break;
            case KeyEvent.VK_ALT: alt = true; break;
            case KeyEvent.VK_SHIFT: shift = true; break;
        }
    }
    public void keyReleased(KeyEvent e) {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_CONTROL: control = false; break;
            case KeyEvent.VK_ALT: alt = false; break;
            case KeyEvent.VK_SHIFT: shift = false; break;
        }
    }

    public void mouseClicked(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }

    int mousex;
    int mousey;
    public void mouseMoved(MouseEvent e) {
        mousex = e.getX();
        mousey = e.getY();
    }

    float tx = 0;
    float ty = 0;
    float tz = 0;
    float anglex = 0;
    float angley = 0;
    public void mouseDragged(MouseEvent e) {
        if (shift) {
            tx += (mousex - e.getX())/(float)20;
            ty += (mousey - e.getY())/(float)20;
        } else {
            anglex -= mousex - e.getX();
            angley += mousey - e.getY();
        }
        mousex = e.getX();
        mousey = e.getY();
    }

    /**
     * Take care of initialization here.
     */
    public void init(GLAutoDrawable gld) {
        GL gl = gld.getGL();
        gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
        gl.glViewport(0, 0, 500, 300);
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glClearDepth(1.0);
        gl.glDepthFunc(GL.GL_LEQUAL);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glMatrixMode(GL.GL_MODELVIEW);
        display(gld);
    }

    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) { }
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) { }
    public synchronized void display(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        GLU glu = new GLU();
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glPointSize(5.0f);
        gl.glLoadIdentity();
        glu.gluPerspective(50-tz, ((float)drawable.getWidth())/drawable.getHeight(), 0.5, 10);
        glu.gluLookAt(0, 0, -1, 0, 0, 0, 0, 1, 0);
        gl.glTranslatef(tx/(float)20, ty/(float)20, 0);
        gl.glRotatef(anglex/3, 0, 1, 0);
        gl.glRotatef(angley/3, 1, 0, 0);

        gl.glBegin(GL.GL_TRIANGLES);
        draw(gl, true, tile);
        gl.glEnd();

        gl.glBegin(GL.GL_TRIANGLES);
        gl.glColor4f((float)0.5, (float)0.5, (float)0.5, (float)0.8);
        //draw(gl, false, goal);
        gl.glEnd();


        int i = 0;
        //gl.glDisable(GL.GL_DEPTH_TEST);
        gl.glColor4f(1,1,1,1);
        for(Matrix m : translations) {
            //if (v1.z==0 && v1.y==0) continue;
            i++;
            if (i != 1 /*&& i!=4*/) continue;
            Point p = new Point(0, 0, 0).times(m);
            Vec v = new Vec(p.x, p.y, p.z);
            v = v.times((float)1.04);
            gl.glTranslatef(v.x, v.y, v.z);
            draw(gl, false, tile);
            gl.glTranslatef(-v.x, -v.y, -v.z);
        }
        //gl.glEnable(GL.GL_DEPTH_TEST);
    }

    private synchronized void draw(GL gl, boolean triangles, Mesh mesh) {
        float red = 0.0f;
        float green = 0.0f;
        float blue = 0.0f;
        for(Mesh.T t : mesh) {
            if (red < 0.15) red = 1.0f;
            if (green < 0.15) green = 1.0f;
            if (blue < 0.15) blue = 1.0f;
            red -= .09f;
            green -= .12f;
            blue -= .15f;

            if (triangles) switch(t.color) {
                case 0: gl.glColor4f((float)0.25, (float)0.25, (float)0.75, (float)0.3); break;
                case 1: gl.glColor4f((float)0.25, (float)0.75, (float)0.25, (float)0.3); break;
                case 2: gl.glColor4f((float)0.75, (float)0.25, (float)0.25, (float)0.3); break;
                case 3: gl.glColor4f((float)0.50, (float)0.50, (float)0.50, (float)0.3); break;
            }
            //gl.glBegin(GL.GL_LINES);

            if (triangles) {
                gl.glBegin(GL.GL_TRIANGLES);
                t.glVertices(gl);
                gl.glEnd();
            } else {
                gl.glBegin(GL.GL_LINES);
                t.e1().p1.p.glVertex(gl);
                t.e1().p2.p.glVertex(gl);
                t.e2().p1.p.glVertex(gl);
                t.e2().p2.p.glVertex(gl);
                t.e3().p1.p.glVertex(gl);
                t.e3().p2.p.glVertex(gl);
                gl.glEnd();
            }

            Point centroid = t.centroid();
            gl.glBegin(GL.GL_LINES);
            gl.glColor3f(1, 1, 1);
            /*
            centroid.glVertex(gl);
            centroid.plus(t.norm().times(t.diameter())).glVertex(gl);
            */

            if (mesh==goal)
                for(Mesh.Vert p : new Mesh.Vert[] { t.v1(), t.v2(), t.v3() }) {
                    p.p.glVertex(gl);
                    //p.plus(p.norm().times(p.score()*10)).glVertex(gl);
                    //p.partner().p.glVertex(gl);
                    //tile.nearest(p).centroid().glVertex(gl);
                }

            gl.glEnd();

        }
    }

    private GLCanvas glcanvas;
    public MeshViewer(Frame f) {
        GLCapabilities glcaps = new GLCapabilities();
        glcanvas = new GLCanvas();
        glcanvas.addGLEventListener(this);
        f.add(glcanvas, BorderLayout.CENTER);
        f.pack();
        f.show();
        f.setSize(900, 900);
        f.doLayout();
        glcanvas.addMouseListener(this);
        glcanvas.addMouseMotionListener(this);
        glcanvas.addMouseWheelListener(this);
        glcanvas.addKeyListener(this);
    }
    public void repaint() {
        glcanvas.repaint();
    }
}